﻿using Steamworks;
using System;
using System.Collections;
using UnityEngine;
using HMLLibrary;

public class DynamiteStick : MonoBehaviour
{
    public static Item_Base item;

    public static double delay = 1.0f;
    public DateTime latestThrow;
    public static float trust = 750f;

    public static GameObject dynamiteStickPrefab;
    public static GameObject explosionEffectPrefab;

    public static AudioClip explosionAudioClip;
    public static AudioClip fuseAudioClip;

    public static async void Initialize()
    {
        item = await WeaponsMod.assetbundle.TaskLoadAssetAsync<Item_Base>("Dynamite_Item_Base");
        RAPI.RegisterItem(item);
        GameObject gobject = await WeaponsMod.assetbundle.TaskLoadAssetAsync<GameObject>("Dynamite_Prefab");
        DynamiteStick script = gobject.AddComponent<DynamiteStick>();
        RAPI.SetItemObject(item, gobject);

        dynamiteStickPrefab = await WeaponsMod.assetbundle.TaskLoadAssetAsync<GameObject>("ThrowableDynamite");
        dynamiteStickPrefab.AddComponent<ThrowableDynamite>();
        explosionEffectPrefab = await WeaponsMod.assetbundle.TaskLoadAssetAsync<GameObject>("DynamiteExplosionPrefab");

        explosionAudioClip = await WeaponsMod.assetbundle.TaskLoadAssetAsync<AudioClip>("dynamite_explosion");
        fuseAudioClip = await WeaponsMod.assetbundle.TaskLoadAssetAsync<AudioClip>("dynamite_fuse");
    }

    public void Update()
    {
        transform.localPosition = new Vector3(0.09f, -0.01f, 0f);
        transform.localRotation = Quaternion.Euler(90f, 0f, 90f);
        if (CanvasHelper.ActiveMenu != MenuType.None || !GetComponentInParent<Network_Player>().IsLocalPlayer)
        {
            return;
        }
        if (MyInput.GetButtonDown("LMB") && latestThrow.AddSeconds(delay) < DateTime.Now)
        {
            latestThrow = DateTime.Now;
            ThrowDynamite();
            RAPI.GetLocalPlayer().Inventory.RemoveItem("WeaponsMod_Dynamite", 1);
        }
    }

    public void ThrowDynamite()
    {
        MessageHandler.SendMessage(new WMessage_ThrowDynamite((Messages)WeaponsMod.NetworkMessages.WeaponsMod_ThrowDynamite, SteamUser.GetSteamID(), transform.position, transform.rotation, RAPI.GetLocalPlayer().Camera.transform.forward));

        GameObject tnt = Instantiate(dynamiteStickPrefab, transform.position, transform.rotation, null);
        tnt.GetComponent<Rigidbody>().AddForce(RAPI.GetLocalPlayer().Camera.transform.forward * trust);
        AudioSource audiosource = tnt.AddComponent<AudioSource>();
        audiosource.spatialBlend = 1;
        audiosource.clip = fuseAudioClip;
        audiosource.Play();
    }
}

public class ThrowableDynamite : MonoBehaviour
{
    public static float secondsBeforeExplosionAfterCollision = 4.5f;

    public bool effectsOnly = false;

    bool isInAir = true;
    Rigidbody rb;
    float explosionRadius = 3f;

    void Start()
    {
        rb = gameObject.GetComponent<Rigidbody>();
    }

    public void Update()
    {
        if (isInAir && rb.velocity.magnitude < 1)
        {
            isInAir = false;
            if (Vector3.Distance(transform.position, BlockCreator.ClosestFoundationToPosition(transform.position).transform.position) < 3.5f)
            {
                transform.SetParentSafe(ComponentManager<Raft>.Value.transform);
            }
            StartCoroutine(ExplosionTimer());
        }
    }

    public IEnumerator ExplosionTimer()
    {
        yield return new WaitForSeconds(secondsBeforeExplosionAfterCollision);
        AudioSource audiosource = gameObject.GetComponent<AudioSource>();
        audiosource.Stop();
        DynamiteStick.explosionEffectPrefab.transform.localScale = new Vector3(5, 5, 5);
        GameObject explosion = Instantiate(DynamiteStick.explosionEffectPrefab, gameObject.transform.position, Quaternion.identity);
        explosion.AddComponent<AudioSource>();
        explosion.GetComponent<AudioSource>().clip = DynamiteStick.explosionAudioClip;
        explosion.GetComponent<AudioSource>().spatialBlend = 1;
        explosion.GetComponent<AudioSource>().Play();
        Destroy(explosion, 5f);
        if (!effectsOnly)
        {
            yield return new WaitForSeconds(0.1f);
            foreach (MeshRenderer t in gameObject.GetComponentsInChildren<MeshRenderer>())
            {
                t.enabled = false;
            }

            foreach (Stat_Health p in FindObjectsOfType<Stat_Health>())
            {
                if (Vector3.Distance(p.transform.position, gameObject.transform.position) <= explosionRadius)
                {
                    p.Value = 0;
                }
            }

            foreach (Block block in FindObjectsOfType<Block>())
            {
                if (Vector3.Distance(block.transform.position, gameObject.transform.position) <= explosionRadius)
                {
                    if (Raft_Network.IsHost)
                    {
                        RAPI.GetLocalPlayer().Network.RPC(new Message_BlockCreator_RemoveBlock(Messages.BlockCreator_RemoveBlock, RAPI.GetLocalPlayer().BlockCreator, RAPI.GetLocalPlayer().steamID, block,true), Target.Other, EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                        WeaponsMod.RemoveBlock(block);
                        continue;
                    }
                    RAPI.GetLocalPlayer().Network.SendP2P(RAPI.GetLocalPlayer().Network.HostID, new Message_BlockCreator_RemoveBlock(Messages.BlockCreator_RemoveBlock, RAPI.GetLocalPlayer().BlockCreator, RAPI.GetLocalPlayer().steamID, block, true), EP2PSend.k_EP2PSendReliable, NetworkChannel.Channel_Game);
                }
            }
        }
        Destroy(gameObject);
    }
}