﻿using Sirenix.Serialization;
using Steamworks;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using UnityEngine;

public class MessageHandler : MonoBehaviour
{

    public static void SendMessage(Message message)
    {
        if (Raft_Network.IsHost)
        {
            RAPI.GetLocalPlayer().Network.RPC(message, Target.Other, EP2PSend.k_EP2PSendReliable, (NetworkChannel)69);
        }
        else
        {
            RAPI.GetLocalPlayer().SendP2P(message, EP2PSend.k_EP2PSendReliable, (NetworkChannel)69);
        }
    }

    public static void ReadP2P_Channel_WeaponsMod()
    {
        NetworkMessage netMessage = RAPI.ListenForNetworkMessagesOnChannel(69);
        if (netMessage != null)
        {
            CSteamID id = netMessage.steamid;
            Message message = netMessage.message;
            switch (message.Type)
            {
                case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_Shoot:
                    {
                        WMessage_Shoot msg = message as WMessage_Shoot;
                        if (msg.hit)
                        {
                            GameObject impactEffect = Instantiate(WeaponsMod.assetbundle.LoadAsset<GameObject>(msg.GroundParticleEffect), msg.hitPosition, Quaternion.Euler(msg.hitRotation), null);
                            Destroy(impactEffect, 2f);
                        }

                        Raft_Network network = ComponentManager<Raft_Network>.Value;
                        Network_Player player = network.GetPlayerFromID(msg.player);
                        GameObject sound = new GameObject("WeaponShootSound");
                        sound.transform.position = player.transform.position;
                        sound.AddComponent<AudioSource>().clip = WeaponsMod.assetbundle.LoadAsset<AudioClip>(msg.shootSound);
                        sound.GetComponent<AudioSource>().spatialBlend = 1;
                        sound.GetComponent<AudioSource>().Play();
                        Destroy(sound, 5f);
                        GameObject currentItem = player.rightHandParent.transform.Find("ModdedItem_" + msg.currentItem).gameObject;
                        if (currentItem.activeSelf)
                        {
                            GameObject muzzleflash = Instantiate(WeaponsMod.assetbundle.LoadAsset<GameObject>(msg.MuzzleParticleEffect), currentItem.transform.Find("Gun").Find("ShootingPoint").Find("Muzzleflash"));
                            muzzleflash.transform.localPosition = new Vector3(0, 0, 0);
                            muzzleflash.transform.localRotation = Quaternion.Euler(0, 0, 0);
                            Destroy(muzzleflash, 5f);
                        }
                        break;
                    }
                case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_EmptyClip:
                    {
                        WMessage_EmptyClip msg = message as WMessage_EmptyClip;
                        GameObject sound = new GameObject("EmptyClipSound");
                        sound.transform.position = msg.position;
                        sound.AddComponent<AudioSource>().clip = WeaponsMod.emptyAudioClip;
                        sound.GetComponent<AudioSource>().spatialBlend = 1;
                        sound.GetComponent<AudioSource>().Play();
                        Destroy(sound, 2f);
                        break;
                    }
                case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_Reload:
                    {
                        WMessage_Reload msg = message as WMessage_Reload;
                        GameObject sound = new GameObject("WeaponReloadSound");
                        sound.transform.position = msg.position;
                        sound.AddComponent<AudioSource>().clip = WeaponsMod.assetbundle.LoadAsset<AudioClip>(msg.soundName);
                        sound.GetComponent<AudioSource>().spatialBlend = 1;
                        sound.GetComponent<AudioSource>().Play();
                        Destroy(sound, 5f);
                        break;
                    }
                case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_StartAiming:
                    {
                        WMessage_StartAiming msg = message as WMessage_StartAiming;
                        Raft_Network network = ComponentManager<Raft_Network>.Value;
                        Network_Player player = network.GetPlayerFromID(msg.player);
                        GameObject currentItem = player.rightHandParent.transform.Find("ModdedItem_" + msg.currentItem).gameObject;
                        if (currentItem.activeSelf && currentItem.GetComponent(msg.script) != null)
                        {
                            WeaponBase weapon = currentItem.GetComponent(msg.script) as WeaponBase;
                            if (weapon != null)
                            {
                                weapon.Aiming = true;
                            }
                        }
                        break;
                    }
                case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_StopAiming:
                    {
                        WMessage_StopAiming msg = message as WMessage_StopAiming;
                        Raft_Network network = ComponentManager<Raft_Network>.Value;
                        Network_Player player = network.GetPlayerFromID(msg.player);
                        GameObject currentItem = player.rightHandParent.transform.Find("ModdedItem_" + msg.currentItem).gameObject;
                        if (currentItem.activeSelf && currentItem.GetComponent(msg.script) != null)
                        {
                            WeaponBase weapon = currentItem.GetComponent(msg.script) as WeaponBase;
                            if (weapon != null)
                            {
                                weapon.Aiming = false;
                            }
                        }
                        break;
                    }
                case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_ThrowDynamite:
                    {
                        WMessage_ThrowDynamite msg = message as WMessage_ThrowDynamite;
                        Raft_Network network = ComponentManager<Raft_Network>.Value;
                        Network_Player player = network.GetPlayerFromID(msg.player);

                        GameObject tnt = Instantiate(DynamiteStick.dynamiteStickPrefab, msg.throwPosition, msg.throwRotation, null);
                        tnt.GetComponent<ThrowableDynamite>().effectsOnly = true;
                        tnt.GetComponent<Rigidbody>().AddForce(msg.throwDirection * DynamiteStick.trust);
                        AudioSource audiosource = tnt.AddComponent<AudioSource>();
                        audiosource.spatialBlend = 1;
                        audiosource.clip = DynamiteStick.fuseAudioClip;
                        audiosource.Play();
                        break;
                    }
                case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_ThrowSmoke:
                    {
                        WMessage_ThrowSmoke msg = message as WMessage_ThrowSmoke;
                        Raft_Network network = ComponentManager<Raft_Network>.Value;
                        Network_Player player = network.GetPlayerFromID(msg.player);

                        GameObject smokePrefab = Instantiate(WeaponsMod.assetbundle.LoadAsset<GameObject>("SmokeGrenade" + msg.color + "_ThrowablePrefab"), msg.throwPosition, msg.throwRotation, null);
                        smokePrefab.AddComponent<ThrowableSmoke>();
                        smokePrefab.GetComponent<Rigidbody>().AddForce(msg.throwDirection * SmokeGrenade.trust);
                        AudioSource audiosource = smokePrefab.AddComponent<AudioSource>();
                        audiosource.spatialBlend = 1;
                        audiosource.clip = SmokeGrenade.fuseAudioClip;
                        audiosource.Play();
                        break;
                    }
            }
        }
        /*
        uint num;
        while (SteamNetworking.IsP2PPacketAvailable(out num, 69))
        {
            byte[] array = new byte[num];
            uint num2;
            CSteamID csteamID;
            if (SteamNetworking.ReadP2PPacket(array, num, out num2, out csteamID, 69))
            {
                Packet packet = SerializationUtility.DeserializeValue<Packet>(array, DataFormat.Binary, new DeserializationContext() { Binder = new TwoWayPreMergeToMergedDeserializationBinder() });
                Packet_Multiple packet_Multiple;
                if (packet.PacketType == PacketType.Single)
                {
                    Packet_Single packet_Single = packet as Packet_Single;
                    packet_Multiple = new Packet_Multiple(packet_Single.SendType);
                    packet_Multiple.messages = new Message[]
                    {
                        packet_Single.message
                    };
                }
                else
                {
                    packet_Multiple = (packet as Packet_Multiple);
                }
                List<Message> list = packet_Multiple.messages.ToList<Message>();
                for (int i = 0; i < list.Count; i++)
                {
                    Message message = list[i];
                    if (message != null)
                    {
                        Messages type = message.Type;
                        switch (type)
                        {
                            case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_Shoot:
                                {
                                    WMessage_Shoot msg = message as WMessage_Shoot;

                                    if (msg.hit)
                                    {
                                        GameObject impactEffect = Instantiate(WeaponsMod.assetbundle.LoadAsset<GameObject>(msg.GroundParticleEffect), msg.hitPosition, Quaternion.Euler(msg.hitRotation), null);
                                        Destroy(impactEffect, 2f);
                                    }

                                    Raft_Network network = ComponentManager<Raft_Network>.Value;
                                    Network_Player player = network.GetPlayerFromID(msg.player);
                                    GameObject sound = new GameObject("WeaponShootSound");
                                    sound.transform.position = player.transform.position;
                                    sound.AddComponent<AudioSource>().clip = WeaponsMod.assetbundle.LoadAsset<AudioClip>(msg.shootSound);
                                    sound.GetComponent<AudioSource>().spatialBlend = 1;
                                    sound.GetComponent<AudioSource>().Play();
                                    Destroy(sound, 5f);
                                    GameObject currentItem = player.rightHandParent.transform.Find("ModdedItem_" + msg.currentItem).gameObject;
                                    if (currentItem.activeSelf)
                                    {
                                        GameObject muzzleflash = Instantiate(WeaponsMod.assetbundle.LoadAsset<GameObject>(msg.MuzzleParticleEffect), currentItem.transform.Find("Gun").Find("ShootingPoint").Find("Muzzleflash"));
                                        muzzleflash.transform.localPosition = new Vector3(0, 0, 0);
                                        muzzleflash.transform.localRotation = Quaternion.Euler(0, 0, 0);
                                        Destroy(muzzleflash, 5f);
                                    }
                                    break;
                                }
                            case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_EmptyClip:
                                {
                                    WMessage_EmptyClip msg = message as WMessage_EmptyClip;
                                    GameObject sound = new GameObject("EmptyClipSound");
                                    sound.transform.position = msg.position;
                                    sound.AddComponent<AudioSource>().clip = WeaponsMod.emptyAudioClip;
                                    sound.GetComponent<AudioSource>().spatialBlend = 1;
                                    sound.GetComponent<AudioSource>().Play();
                                    Destroy(sound, 2f);
                                    break;
                                }
                            case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_Reload:
                                {
                                    WMessage_Reload msg = message as WMessage_Reload;
                                    GameObject sound = new GameObject("WeaponReloadSound");
                                    sound.transform.position = msg.position;
                                    sound.AddComponent<AudioSource>().clip = WeaponsMod.assetbundle.LoadAsset<AudioClip>(msg.soundName);
                                    sound.GetComponent<AudioSource>().spatialBlend = 1;
                                    sound.GetComponent<AudioSource>().Play();
                                    Destroy(sound, 5f);
                                    break;
                                }
                            case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_StartAiming:
                                {
                                    WMessage_StartAiming msg = message as WMessage_StartAiming;
                                    Raft_Network network = ComponentManager<Raft_Network>.Value;
                                    Network_Player player = network.GetPlayerFromID(msg.player);
                                    GameObject currentItem = player.rightHandParent.transform.Find("ModdedItem_" + msg.currentItem).gameObject;
                                    if (currentItem.activeSelf && currentItem.GetComponent(msg.script) != null)
                                    {
                                        WeaponBase weapon = currentItem.GetComponent(msg.script) as WeaponBase;
                                        if (weapon != null)
                                        {
                                            weapon.Aiming = true;
                                        }
                                    }
                                    break;
                                }
                            case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_StopAiming:
                                {
                                    WMessage_StopAiming msg = message as WMessage_StopAiming;
                                    Raft_Network network = ComponentManager<Raft_Network>.Value;
                                    Network_Player player = network.GetPlayerFromID(msg.player);
                                    GameObject currentItem = player.rightHandParent.transform.Find("ModdedItem_" + msg.currentItem).gameObject;
                                    if (currentItem.activeSelf && currentItem.GetComponent(msg.script) != null)
                                    {
                                        WeaponBase weapon = currentItem.GetComponent(msg.script) as WeaponBase;
                                        if (weapon != null)
                                        {
                                            weapon.Aiming = false;
                                        }
                                    }
                                    break;
                                }
                            case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_ThrowDynamite:
                                {
                                    WMessage_ThrowDynamite msg = message as WMessage_ThrowDynamite;
                                    Raft_Network network = ComponentManager<Raft_Network>.Value;
                                    Network_Player player = network.GetPlayerFromID(msg.player);

                                    GameObject tnt = Instantiate(DynamiteStick.dynamiteStickPrefab, msg.throwPosition, msg.throwRotation, null);
                                    tnt.GetComponent<ThrowableDynamite>().effectsOnly = true;
                                    tnt.GetComponent<Rigidbody>().AddForce(msg.throwDirection * DynamiteStick.trust);
                                    AudioSource audiosource = tnt.AddComponent<AudioSource>();
                                    audiosource.spatialBlend = 1;
                                    audiosource.clip = DynamiteStick.fuseAudioClip;
                                    audiosource.Play();
                                    break;
                                }
                            case (Messages)WeaponsMod.NetworkMessages.WeaponsMod_ThrowSmoke:
                                {
                                    WMessage_ThrowSmoke msg = message as WMessage_ThrowSmoke;
                                    Raft_Network network = ComponentManager<Raft_Network>.Value;
                                    Network_Player player = network.GetPlayerFromID(msg.player);

                                    GameObject smokePrefab = Instantiate(WeaponsMod.assetbundle.LoadAsset<GameObject>("SmokeGrenade" + msg.color + "_ThrowablePrefab"), msg.throwPosition, msg.throwRotation, null);
                                    smokePrefab.AddComponent<ThrowableSmoke>();
                                    smokePrefab.GetComponent<Rigidbody>().AddForce(msg.throwDirection * SmokeGrenade.trust);
                                    AudioSource audiosource = smokePrefab.AddComponent<AudioSource>();
                                    audiosource.spatialBlend = 1;
                                    audiosource.clip = SmokeGrenade.fuseAudioClip;
                                    audiosource.Play();
                                    break;
                                }
                        }
                    }

                }
            }
        }*/
    }
}