﻿using System;
using System.Linq;
using UnityEngine;
using HMLLibrary;

public class TripleBarrel : WeaponBase
{
    public static Item_Base item;

    public static async void Initialize()
    {
        item = await WeaponsMod.assetbundle.TaskLoadAssetAsync<Item_Base>("TripleBarrel_Item_Base");
        RAPI.RegisterItem(item);
        GameObject gobject = await WeaponsMod.assetbundle.TaskLoadAssetAsync<GameObject>("TripleBarrel_Prefab");
        TripleBarrel script = gobject.AddComponent<TripleBarrel>();

        // Set default values
        script.currentItem = item;
        script.shootAudioClip = await WeaponsMod.assetbundle.TaskLoadAssetAsync<AudioClip>("m4shotgun_shoot");
        script.reloadAudioClip = await WeaponsMod.assetbundle.TaskLoadAssetAsync<AudioClip>("m4shotgun_reload");
        script.emptyClipAudioClip = WeaponsMod.emptyAudioClip;
        script.muzzleflashPrefab = await WeaponsMod.assetbundle.TaskLoadAssetAsync<GameObject>("ShotgunMuzzleFlash");

        script.shootingPoint = gobject.transform.Find("Gun").Find("ShootingPoint").gameObject;
        script.muzzleFlash = script.shootingPoint.transform.Find("Muzzleflash").gameObject;

        script.ammoType = ItemManager.GetItemByName("WeaponsMod_ShotgunAmmo");
        script.idle_weapon_position = new Vector3(0.09f, -0.01f, -0.05f);
        script.idle_weapon_rotation = Quaternion.Euler(90f, 0f, 90f);
        script.aiming_weapon_position = new Vector3(0, 0, -0.20f);
        script.aiming_weapon_rotation = Quaternion.Euler(90, 0, 0);
        script.clipSize = 3;
        script.damagePerShot = 20;
        script.range = 8f;
        script.shootingDelay = 0.6f;
        script.reloadDelay = 3500f;

        RAPI.SetItemObject(item, gobject);
    }

    public override void PostAiming()
    {
        if (parentTransform == null)
        {
            parentTransform = transform.GetComponentInParent<PlayerAnimator>().transform;
            return;
        }

        if (!skeletonObjects_aiming.ContainsKey("ORG-hand_R"))
        {
            skeletonObjects_aiming.Add("ORG-hand_R", new TransformRotAnimation(parentTransform.FindChildRecursively("ORG-hand_R"), Quaternion.Euler(355, 325, 354)));
            return;
        }

        if (!skeletonObjects_aiming.ContainsKey("ORG-upper_arm_L"))
        {
            skeletonObjects_aiming.Add("ORG-upper_arm_L", new TransformRotAnimation(parentTransform.FindChildRecursively("ORG-upper_arm_L"), Quaternion.Euler(80, 111, 178)));
            return;
        }

        if (!skeletonObjects_aiming.ContainsKey("ORG-forearm_R"))
        {
            skeletonObjects_aiming.Add("ORG-forearm_R", new TransformRotAnimation(parentTransform.FindChildRecursively("ORG-forearm_R"), Quaternion.Euler(352, 45, 0)));
            return;
        }

        skeletonObjects_aiming.ToList().ForEach(k =>
        {
            k.Value.transform.localRotation = k.Value.rotation;
        });

    }

    public override void PostIdling()
    {
        if (parentTransform == null)
        {
            parentTransform = transform.GetComponentInParent<PlayerAnimator>().transform;
            return;
        }

        skeletonObjects_idling.ToList().ForEach(k =>
        {
            k.Value.transform.localRotation = k.Value.rotation;
        });
    }
}