﻿using Steamworks;
using System;
using System.Collections.Generic;
using UnityEngine;

public class WeaponBase : MonoBehaviour
{
    // Public variables
    public Item_Base currentItem = null;
    public AudioClip shootAudioClip;
    public AudioClip reloadAudioClip;
    public AudioClip emptyClipAudioClip;
    public GameObject muzzleflashPrefab;
    public int clipSize = 0;
    public int damagePerShot = 0;
    public float range = 0f;
    public double shootingDelay = 0f;
    public float reloadDelay = 0f;
    public Item_Base ammoType = ItemManager.GetItemByName("Planks");
    public GameObject shootingPoint;
    public GameObject muzzleFlash;
    public Vector3 idle_weapon_position = new Vector3(0, 0, 0);
    public Quaternion idle_weapon_rotation = Quaternion.Euler(0, 0, 0);
    public Vector3 aiming_weapon_position = new Vector3(0, 0, 0);
    public Quaternion aiming_weapon_rotation = Quaternion.Euler(0, 0, 0);
    public bool CanAim = true;
    public PlayerAnimation idleAnimation = PlayerAnimation.Index_5_HoldItem;
    public PlayerAnimation aimingAnimation = PlayerAnimation.Index_17_Bow;
    public bool Aiming = false;
    public Transform parentTransform;

    // Private variables
    private bool isReloading;
    private DateTime lastshoot;
    private DateTime lastReload;

    public Dictionary<string, TransformRotAnimation> skeletonObjects_aiming = new Dictionary<string, TransformRotAnimation>();
    public Dictionary<string, TransformRotAnimation> skeletonObjects_idling = new Dictionary<string, TransformRotAnimation>();

    bool isLocalPlayer = false;

    void Start()
    {
        isLocalPlayer = GetComponentInParent<Network_Player>().IsLocalPlayer;
    }

    void OnEnable()
    {
        isLocalPlayer = GetComponentInParent<Network_Player>().IsLocalPlayer;
    }

    public void OnSelect()
    {
        if (!isLocalPlayer) { return; }
        Network_Player player = RAPI.GetLocalPlayer();
        Aiming = false;
        MessageHandler.SendMessage(new WMessage_StopAiming((Messages)WeaponsMod.NetworkMessages.WeaponsMod_StopAiming, SteamUser.GetSteamID(), currentItem.UniqueName, GetType().Name));
        if (player.Inventory.GetSelectedHotbarItem().UniqueIndex == currentItem.UniqueIndex && player.Inventory.GetSelectedHotbarItem().exclusiveString == "")
        {
            player.Inventory.GetSelectedHotbarItem().exclusiveString = "0";
        }
    }

    public void OnDeSelect()
    {
        if (!isLocalPlayer) { return; }
        Aiming = false;
        MessageHandler.SendMessage(new WMessage_StopAiming((Messages)WeaponsMod.NetworkMessages.WeaponsMod_StopAiming, SteamUser.GetSteamID(), currentItem.UniqueName, GetType().Name));
    }

    public virtual void PostAiming()
    {
    }

    public virtual void PostIdling()
    {
    }

    public void LateUpdate()
    {
        if (Aiming)
        {
            transform.localPosition = aiming_weapon_position;
            transform.localRotation = aiming_weapon_rotation;
            GetComponentInParent<PlayerAnimator>().SetAnimation(aimingAnimation);
            PostAiming();
        }
        else
        {
            transform.localPosition = idle_weapon_position;
            transform.localRotation = idle_weapon_rotation;
            GetComponentInParent<PlayerAnimator>().SetAnimation(idleAnimation);
            PostIdling();
        }

        if (!isLocalPlayer)
        {
            return;
        }

        if (CanAim)
        {
            if (Input.GetMouseButton(1) && !isReloading)
            {
                if (!Aiming)
                {
                    Aiming = true;
                    MessageHandler.SendMessage(new WMessage_StartAiming((Messages)WeaponsMod.NetworkMessages.WeaponsMod_StartAiming, SteamUser.GetSteamID(), currentItem.UniqueName, GetType().Name));
                }
            }
            else
            {
                if (Aiming)
                {
                    Aiming = false;
                    MessageHandler.SendMessage(new WMessage_StopAiming((Messages)WeaponsMod.NetworkMessages.WeaponsMod_StopAiming, SteamUser.GetSteamID(), currentItem.UniqueName, GetType().Name));
                }
            }
        }

        if (CanvasHelper.ActiveMenu != MenuType.None || !GetComponentInParent<Network_Player>().IsLocalPlayer || RAPI.GetLocalPlayer() == null || RAPI.GetLocalPlayer().Inventory == null)
        {
            return;
        }

        if (isReloading)
        {
            RAPI.GetLocalPlayer().Animator.SetAnimation(PlayerAnimation.Index_0_Idle);
            if (lastReload.AddMilliseconds(reloadDelay) < DateTime.Now && isReloading)
            {
                isReloading = false;
                int missingBulletsInClip = clipSize - int.Parse(RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().exclusiveString);
                if (RAPI.GetLocalPlayer().Inventory.GetItemCount(ammoType.UniqueName) > 0 && RAPI.GetLocalPlayer().Inventory.GetItemCount(ammoType.UniqueName) < clipSize)
                {
                    int amount = RAPI.GetLocalPlayer().Inventory.GetItemCount(ammoType.UniqueName);
                    RAPI.GetLocalPlayer().Inventory.RemoveItem(ammoType.UniqueName, amount);
                    RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().exclusiveString = Mathf.Clamp(amount, 0, clipSize).ToString();
                }
                else
                {
                    RAPI.GetLocalPlayer().Inventory.RemoveItem(ammoType.UniqueName, missingBulletsInClip);
                    RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().exclusiveString = Mathf.Clamp(int.Parse(RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().exclusiveString) + missingBulletsInClip, 0, clipSize).ToString();
                }
                PlayerItemManager.IsBusy = false;
            }
            return;
        }
        if (Input.GetKeyDown(KeyCode.R) && int.Parse(RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().exclusiveString) < clipSize && RAPI.GetLocalPlayer().Inventory.GetItemCount(ammoType.UniqueName) > 0)
        {
            lastReload = DateTime.Now;
            isReloading = true;
            MessageHandler.SendMessage(new WMessage_Reload((Messages)WeaponsMod.NetworkMessages.WeaponsMod_Reload, transform.position, reloadAudioClip.name));
            PlayerItemManager.IsBusy = true;
            if (gameObject.GetComponent<AudioSource>() == null)
            {
                gameObject.AddComponent<AudioSource>().clip = reloadAudioClip;
                gameObject.GetComponent<AudioSource>().playOnAwake = false;
                gameObject.GetComponent<AudioSource>().spatialBlend = 1;
                gameObject.GetComponent<AudioSource>().Play();
            }
            else
            {
                gameObject.GetComponent<AudioSource>().clip = reloadAudioClip;
                gameObject.GetComponent<AudioSource>().playOnAwake = false;
                gameObject.GetComponent<AudioSource>().spatialBlend = 1;
                gameObject.GetComponent<AudioSource>().Play();
            }
            return;
        }
        if (GameModeValueManager.GetCurrentGameModeValue().playerSpecificVariables.unlimitedResources)
        {
            WeaponsMod.AmmoHud_text.text = Mathf.Clamp(int.Parse(RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().exclusiveString), 0, clipSize) + "/∞";
        }
        else
        {
            WeaponsMod.AmmoHud_text.text = Mathf.Clamp(int.Parse(RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().exclusiveString), 0, clipSize) + "/" + RAPI.GetLocalPlayer().Inventory.GetItemCount(ammoType);
        }

        if (Aiming && Input.GetMouseButton(0))
        {
            Shoot();
        }
    }

    void Shoot()
    {
        if (!isLocalPlayer) { return; }
        int clip = int.Parse(RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().exclusiveString);
        if (lastshoot.AddSeconds(shootingDelay) < DateTime.Now)
        {
            lastshoot = DateTime.Now;
            if (GetComponentInChildren<Animation>())
            {
                GetComponentInChildren<Animation>().Stop();
                GetComponentInChildren<Animation>().Play();
            }
            if (clip > 0)
            {
                RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().exclusiveString = (clip - 1).ToString();

                GameObject sound = new GameObject();
                sound.transform.position = RAPI.GetLocalPlayer().transform.position;
                sound.AddComponent<AudioSource>().clip = shootAudioClip;
                sound.GetComponent<AudioSource>().spatialBlend = 1;
                sound.GetComponent<AudioSource>().Play();
                Destroy(sound, 5f);

                GameObject muzzleflash = Instantiate(muzzleflashPrefab, muzzleFlash.transform);
                muzzleflash.transform.localPosition = new Vector3(0, 0, 0);
                muzzleflash.transform.localRotation = Quaternion.Euler(0, 0, 0);
                Destroy(muzzleflash, 5f);

                RaycastHit hit;
                if (Physics.Raycast(RAPI.GetLocalPlayer().Camera.transform.position, RAPI.GetLocalPlayer().Camera.transform.forward, out hit, range, 1 << LayerMask.NameToLayer("Ragdoll") | 1 << LayerMask.NameToLayer("Entity") | 1 << LayerMask.NameToLayer("Block")))
                {
                    Block block = hit.transform.GetComponentInParent<Block>();
                    if (block != null)
                    {
                        block.Damage(damagePerShot * 5);
                        Quaternion rotation = Quaternion.LookRotation(hit.normal);
                        rotation = rotation * Quaternion.Euler(0, 0, 0);
                        GameObject impactEffect = Instantiate(WeaponsMod.GroundImpactEffect, hit.point, rotation, hit.transform);
                        Destroy(impactEffect, 2f);
                        MessageHandler.SendMessage(new WMessage_Shoot((Messages)WeaponsMod.NetworkMessages.WeaponsMod_Shoot, true, shootAudioClip.name, WeaponsMod.GroundImpactEffect.name, muzzleflashPrefab.name, hit.point, rotation.eulerAngles, SteamUser.GetSteamID(), currentItem.UniqueName, GetType().Name));
                        return;
                    }

                    Network_Entity entity = hit.transform.gameObject.GetComponentInParent<Network_Entity>();
                    if (entity == null)
                    {
                        Network_Entity_Redirect entityRedirect = hit.transform.gameObject.GetComponentInParent<Network_Entity_Redirect>();
                        if (entityRedirect != null)
                        {
                            entity = entityRedirect.entity;
                        }
                    }
                    if (entity != null)
                    {
                        ComponentManager<Network_Host>.Value.DamageEntity(entity, hit.transform, damagePerShot * 3, hit.point, hit.normal, EntityType.Player, null);
                        MessageHandler.SendMessage(new WMessage_Shoot((Messages)WeaponsMod.NetworkMessages.WeaponsMod_Shoot, true, shootAudioClip.name, WeaponsMod.GroundImpactEffect.name, muzzleflashPrefab.name, new Vector3(), new Vector3(), SteamUser.GetSteamID(), currentItem.UniqueName, GetType().Name));
                        return;
                    }
                }

                var layerMask = ~((1 << LayerMask.NameToLayer("LocalPlayer")));

                if (Physics.Raycast(RAPI.GetLocalPlayer().Camera.transform.position, RAPI.GetLocalPlayer().Camera.transform.forward, out hit, range, layerMask))
                {
                    GameObject effectPrefab = WeaponsMod.GroundImpactEffect;
                    if (hit.transform.tag == "Water")
                    {
                        effectPrefab = WeaponsMod.WaterImpactEffect;
                    }
                    Quaternion rotation = Quaternion.LookRotation(hit.normal);
                    rotation = rotation * Quaternion.Euler(0, 0, 0);
                    GameObject impactEffect = Instantiate(effectPrefab, hit.point, rotation);
                    MessageHandler.SendMessage(new WMessage_Shoot((Messages)WeaponsMod.NetworkMessages.WeaponsMod_Shoot, true, shootAudioClip.name, effectPrefab.name, muzzleflashPrefab.name, hit.point, rotation.eulerAngles, SteamUser.GetSteamID(), currentItem.UniqueName, GetType().Name));
                    Destroy(impactEffect, 2f);
                    return;
                }
                MessageHandler.SendMessage(new WMessage_Shoot((Messages)WeaponsMod.NetworkMessages.WeaponsMod_Shoot, false, shootAudioClip.name, WeaponsMod.GroundImpactEffect.name, muzzleflashPrefab.name, new Vector3(), new Vector3(), SteamUser.GetSteamID(), currentItem.UniqueName, GetType().Name));
            }
            else
            {
                GameObject sound = new GameObject("Glock18_EmptyClipSound");
                sound.transform.position = RAPI.GetLocalPlayer().transform.position;
                sound.AddComponent<AudioSource>().clip = emptyClipAudioClip;
                sound.GetComponent<AudioSource>().spatialBlend = 1;
                sound.GetComponent<AudioSource>().Play();
                Destroy(sound, 2f);
                MessageHandler.SendMessage(new WMessage_EmptyClip((Messages)WeaponsMod.NetworkMessages.WeaponsMod_EmptyClip, transform.position));
            }
        }

    }
}

public class TransformRotAnimation
{
    public Transform transform;
    public Quaternion rotation;

    public TransformRotAnimation(Transform t, Quaternion r)
    {
        transform = t;
        rotation = r;
    }
}
