﻿using HarmonyLib;
using HMLLibrary;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;
using UnityEngine.UI;

public class WeaponsMod : Mod
{
    public static AssetBundle assetbundle;
    public static GameObject AmmoHud;
    public static Text AmmoHud_text;
    public static Image AmmoHud_icon;
    public static Sprite pistolammo_icon;
    public static Sprite rifleammo_icon;
    public static Sprite shotgunammo_icon;
    public static GameObject GroundImpactEffect;
    public static GameObject WaterImpactEffect;
    public static AudioClip emptyAudioClip;

    public static Item_Base PistolAmmo;
    public static Item_Base RifleAmmo;
    public static Item_Base ShotgunAmmo;

    public enum NetworkMessages
    {
        WeaponsMod_Shoot = 3000,
        WeaponsMod_Reload = 3001,
        WeaponsMod_EmptyClip = 3002,
        WeaponsMod_StartAiming = 3003,
        WeaponsMod_StopAiming = 3004,
        WeaponsMod_ThrowDynamite = 3005,
        WeaponsMod_ThrowSmoke = 3006,
    }

    Harmony HarmonyInstance;

    public async void Start()
    {
        delayWorldLoading = true;
        HNotification notification = FindObjectOfType<HNotify>().AddNotification(HNotify.NotificationType.spinning, "Loading WeaponsMod...");

        assetbundle = await HUtils.TaskLoadAssetBundleFromMemoryAsync(GetEmbeddedFileBytes("weaponsmod.assets"));

        emptyAudioClip = await assetbundle.TaskLoadAssetAsync<AudioClip>("Glock18_empty_clip");
        AmmoHud = Instantiate(await assetbundle.TaskLoadAssetAsync<GameObject>("AmmoHud"), transform);
        AmmoHud_text = AmmoHud.transform.Find("AmmoText").GetComponent<Text>();
        AmmoHud_icon = AmmoHud.transform.Find("AmmoIcon").GetComponent<Image>();
        AmmoHud.SetActive(false);
        pistolammo_icon = await assetbundle.TaskLoadAssetAsync<Sprite>("pistolammo");
        rifleammo_icon = await assetbundle.TaskLoadAssetAsync<Sprite>("rifleammo");
        shotgunammo_icon = await assetbundle.TaskLoadAssetAsync<Sprite>("shotgunammo");
        GroundImpactEffect = await assetbundle.TaskLoadAssetAsync<GameObject>("GroundImpactEffect");
        WaterImpactEffect = await assetbundle.TaskLoadAssetAsync<GameObject>("WaterImpactEffect");
        Item_Base pistolAmmo = await assetbundle.TaskLoadAssetAsync<Item_Base>("PistolAmmo");
        RAPI.RegisterItem(pistolAmmo);
        Item_Base rifleAmmo = await assetbundle.TaskLoadAssetAsync<Item_Base>("RifleAmmo");
        RAPI.RegisterItem(rifleAmmo);
        Item_Base shotgunAmmo = await assetbundle.TaskLoadAssetAsync<Item_Base>("ShotgunAmmo");
        RAPI.RegisterItem(shotgunAmmo);
        Glock18.Initialize();
        Mac11.Initialize();
        DesertEagle.Initialize();
        DynamiteStick.Initialize();
        M4A1.Initialize();
        AK47.Initialize();
        FortniteScar.Initialize();
        SmokeGrenade.Initialize("Blue");
        SmokeGrenade.Initialize("Red");
        SmokeGrenade.Initialize("Green");
        M4Shotgun.Initialize();
        TripleBarrel.Initialize();
        HarmonyInstance = new Harmony("fr.tekgamer.weaponsmod");
        HarmonyInstance.PatchAll(Assembly.GetExecutingAssembly());
        notification.Close();
        delayWorldLoading = false;
    }

    public static void RemoveBlock(Block block)
    {
        List<Block> destroyedBlocks = new List<Block>();
        if (block != null)
        {
            block.gameObject.SetActive(false);
            if (!destroyedBlocks.Contains(block))
            {
                destroyedBlocks.Add(block);
            }
        }

        ((RaftBounds)Traverse.Create(typeof(BlockCreator)).Field("raftBounds").GetValue()).RemovedWalkableBlocks(destroyedBlocks);
        for (int i = 0; i < destroyedBlocks.Count; i++)
        {
            Block block2 = destroyedBlocks[i];
            if (block2 != null)
            {
                if (block2.networkedBehaviour != null)
                {
                    NetworkUpdateManager.RemoveBehaviour(block2.networkedBehaviour);
                }
                DestroyImmediate(block2.gameObject);
            }
        }
    }

    public void Update()
    {
        MessageHandler.ReadP2P_Channel_WeaponsMod();
    }

    public void LateUpdate()
    {
        if (RAPI.IsDedicatedServer()) return;
        if (AmmoHud != null && RAPI.IsCurrentSceneGame() && RAPI.GetLocalPlayer() != null && RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem() != null)
        {
            switch (RAPI.GetLocalPlayer().Inventory.GetSelectedHotbarItem().UniqueName)
            {
                case "WeaponsMod_Glock18":
                    AmmoHud.SetActive(true);
                    AmmoHud_icon.sprite = pistolammo_icon;
                    return;
                case "WeaponsMod_DesertEagle":
                    AmmoHud.SetActive(true);
                    AmmoHud_icon.sprite = pistolammo_icon;
                    return;
                case "WeaponsMod_Mac11":
                    AmmoHud.SetActive(true);
                    AmmoHud_icon.sprite = rifleammo_icon;
                    return;
                case "WeaponsMod_M4A1":
                    AmmoHud.SetActive(true);
                    AmmoHud_icon.sprite = rifleammo_icon;
                    return;
                case "WeaponsMod_AK47":
                    AmmoHud.SetActive(true);
                    AmmoHud_icon.sprite = rifleammo_icon;
                    return;
                case "WeaponsMod_FortniteScar":
                    AmmoHud.SetActive(true);
                    AmmoHud_icon.sprite = rifleammo_icon;
                    return;
                case "WeaponsMod_M4Shotgun":
                    AmmoHud.SetActive(true);
                    AmmoHud_icon.sprite = shotgunammo_icon;
                    return;
                case "WeaponsMod_TripleBarrel":
                    AmmoHud.SetActive(true);
                    AmmoHud_icon.sprite = shotgunammo_icon;
                    return;
            }
        }
        if (AmmoHud != null)
        {
            AmmoHud.SetActive(false);
        }
    }
}

[HarmonyPatch(typeof(Helper))]
[HarmonyPatch("GetTerm")]
static class Localizations
{
    static bool Prefix(Helper __instance, string term, bool applyParameters, ref string __result)
    {
        switch (term)
        {
            case "CraftingSub/WeaponsMod_Ammo":
                __result = "Ammo's";
                return false;
            case "CraftingSub/WeaponsMod_Pistols":
                __result = "Pistols";
                return false;
            case "CraftingSub/WeaponsMod_Rifles":
                __result = "Rifles";
                return false;
            case "CraftingSub/WeaponsMod_Shotguns":
                __result = "Shotguns";
                return false;
            case "CraftingSub/WeaponsMod_Throwables":
                __result = "Throwables";
                return false;
        }
        return true;
    }
}